@@ -27,22 +27,24 @@
  * @see GeanyMainWidgets::message_window_notebook to append a new notebook page.
  **/
 
-#include "geany.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
-#include "support.h"
-#include "prefs.h"
+#include "msgwindow.h"
+
+#include "build.h"
+#include "document.h"
 #include "callbacks.h"
+#include "filetypes.h"
+#include "keybindings.h"
+#include "main.h"
+#include "navqueue.h"
+#include "prefs.h"
+#include "support.h"
 #include "ui_utils.h"
 #include "utils.h"
-#include "document.h"
-#include "filetypes.h"
-#include "build.h"
-#include "main.h"
 #include "vte.h"
-#include "navqueue.h"
-#include "editor.h"
-#include "msgwindow.h"
-#include "keybindings.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -64,6 +66,27 @@ ParseData;
 
 MessageWindow msgwindow;
 
+enum
+{
+	MSG_COL_LINE = 0,
+	MSG_COL_DOC_ID,
+	MSG_COL_COLOR,
+	MSG_COL_STRING,
+	MSG_COL_COUNT
+};
+
+enum
+{
+	COMPILER_COL_COLOR = 0,
+	COMPILER_COL_STRING,
+	COMPILER_COL_COUNT
+};
+
+
+static GdkColor color_error = {0, 0xFFFF, 0, 0};
+static GdkColor color_context = {0, 0x7FFF, 0, 0};
+static GdkColor color_message = {0, 0, 0, 0xD000};
+
 
 static void prepare_msg_tree_view(void);
 static void prepare_status_tree_view(void);
@@ -83,9 +106,12 @@ void msgwin_show_hide_tabs(void)
 }
 
 
-/** Sets the Messages path for opening any parsed filenames without absolute path
- * from message lines.
- * @param messages_dir The directory. **/
+/**
+ * Sets the Messages path for opening any parsed filenames without absolute path from message lines.
+ *
+ * @param messages_dir The directory.
+ **/
+GEANY_API_SYMBOL
 void msgwin_set_messages_dir(const gchar *messages_dir)
 {
 	g_free(msgwindow.messages_dir);
@@ -93,6 +119,37 @@ void msgwin_set_messages_dir(const gchar *messages_dir)
 }
 
 
+static void load_color(const gchar *color_name, GdkColor *color)
+{
+#if GTK_CHECK_VERSION(3, 0, 0)
+	GdkRGBA rgba_color;
+	GtkWidgetPath *path = gtk_widget_path_new();
+	GtkStyleContext *ctx = gtk_style_context_new();
+
+	gtk_widget_path_append_type(path, GTK_TYPE_WINDOW);
+	gtk_widget_path_iter_set_name(path, -1, color_name);
+	gtk_style_context_set_screen(ctx, gdk_screen_get_default());
+	gtk_style_context_set_path(ctx, path);
+	gtk_style_context_get_color(ctx, gtk_style_context_get_state(ctx), &rgba_color);
+
+	color->red   = 0xffff * rgba_color.red;
+	color->green = 0xffff * rgba_color.green;
+	color->blue  = 0xffff * rgba_color.blue;
+
+	gtk_widget_path_unref(path);
+	g_object_unref(ctx);
+#else
+	gchar *path = g_strconcat("*.", color_name, NULL);
+
+	GtkSettings *settings = gtk_settings_get_default();
+	GtkStyle *style = gtk_rc_get_style_by_paths(settings, path, NULL, GTK_TYPE_WIDGET);
+	*color = style->fg[GTK_STATE_NORMAL];
+
+	g_free(path);
+#endif
+}
+
+
 void msgwin_init(void)
 {
 	msgwindow.notebook = ui_lookup_widget(main_widgets.window, "notebook_info");
@@ -111,6 +168,10 @@ void msgwin_init(void)
 
 	ui_widget_modify_font_from_string(msgwindow.scribble, interface_prefs.msgwin_font);
 	g_signal_connect(msgwindow.scribble, "populate-popup", G_CALLBACK(on_scribble_populate), NULL);
+
+	load_color("geany-compiler-error", &color_error);
+	load_color("geany-compiler-context", &color_context);
+	load_color("geany-compiler-message", &color_message);
 }
 
 
@@ -175,15 +236,15 @@ static void prepare_msg_tree_view(void)
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *selection;
 
-	/* line, doc, fg, str */
-	msgwindow.store_msg = gtk_list_store_new(4, G_TYPE_INT, G_TYPE_POINTER,
+	/* line, doc id, fg, str */
+	msgwindow.store_msg = gtk_list_store_new(MSG_COL_COUNT, G_TYPE_INT, G_TYPE_UINT,
 		GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_msg), GTK_TREE_MODEL(msgwindow.store_msg));
 	g_object_unref(msgwindow.store_msg);
 
 	renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
-		"foreground-gdk", 2, "text", 3, NULL);
+		"foreground-gdk", MSG_COL_COLOR, "text", MSG_COL_STRING, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(msgwindow.tree_msg), column);
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_msg), FALSE);
@@ -214,12 +275,13 @@ static void prepare_compiler_tree_view(void)
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *selection;
 
-	msgwindow.store_compiler = gtk_list_store_new(2, GDK_TYPE_COLOR, G_TYPE_STRING);
+	msgwindow.store_compiler = gtk_list_store_new(COMPILER_COL_COUNT, GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_compiler), GTK_TREE_MODEL(msgwindow.store_compiler));
 	g_object_unref(msgwindow.store_compiler);
 
 	renderer = gtk_cell_renderer_text_new();
-	column = gtk_tree_view_column_new_with_attributes(NULL, renderer, "foreground-gdk", 0, "text", 1, NULL);
+	column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
+		"foreground-gdk", COMPILER_COL_COLOR, "text", COMPILER_COL_STRING, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(msgwindow.tree_compiler), column);
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_compiler), FALSE);
@@ -242,31 +304,30 @@ static void prepare_compiler_tree_view(void)
 	/*g_signal_connect(selection, "changed", G_CALLBACK(on_msg_tree_selection_changed), NULL);*/
 }
 
-
-static const GdkColor color_error = {0, 65535, 0, 0};
-
 static const GdkColor *get_color(gint msg_color)
 {
-	static const GdkColor dark_red = {0, 65535 / 2, 0, 0};
-	static const GdkColor blue = {0, 0, 0, 0xD000};	/* not too bright ;-) */
-
 	switch (msg_color)
 	{
 		case COLOR_RED: return &color_error;
-		case COLOR_DARK_RED: return &dark_red;
-		case COLOR_BLUE: return &blue;
+		case COLOR_DARK_RED: return &color_context;
+		case COLOR_BLUE: return &color_message;
 		default: return NULL;
 	}
 }
 
 
 /**
- *  Adds a new message in the compiler tab treeview in the messages window.
+ * Adds a formatted message in the compiler tab treeview in the messages window.
  *
- *  @param msg_color A color to be used for the text. It must be an element of #MsgColors.
- *  @param format @c printf()-style format string.
- *  @param ... Arguments for the @c format string.
+ * @param msg_color A color to be used for the text. It must be an element of #MsgColors.
+ * @param format    @c printf()-style format string.
+ * @param ...       Arguments for the @c format string.
+ *
+ * @see msgwin_compiler_add_string()
+ *
+ * @since 0.16
  **/
+GEANY_API_SYMBOL
 void msgwin_compiler_add(gint msg_color, const gchar *format, ...)
 {
 	gchar *string;
@@ -279,11 +340,20 @@ void msgwin_compiler_add(gint msg_color, const gchar *format, ...)
 	g_free(string);
 }
 
-
+/**
+ * Adds a new message in the compiler tab treeview in the messages window.
+ *
+ * @param msg_color A color to be used for the text. It must be an element of #MsgColors.
+ * @param msg       Compiler message to be added.
+ *
+ * @see msgwin_compiler_add()
+ *
+ * @since 1.34 (API 236)
+ **/
+GEANY_API_SYMBOL
 void msgwin_compiler_add_string(gint msg_color, const gchar *msg)
 {
 	GtkTreeIter iter;
-	GtkTreePath *path;
 	const GdkColor *color = get_color(msg_color);
 	gchar *utf8_msg;
 
@@ -293,12 +363,14 @@ void msgwin_compiler_add_string(gint msg_color, const gchar *msg)
 		utf8_msg = (gchar *) msg;
 
 	gtk_list_store_append(msgwindow.store_compiler, &iter);
-	gtk_list_store_set(msgwindow.store_compiler, &iter, 0, color, 1, utf8_msg, -1);
+	gtk_list_store_set(msgwindow.store_compiler, &iter,
+		COMPILER_COL_COLOR, color, COMPILER_COL_STRING, utf8_msg, -1);
 
 	if (ui_prefs.msgwindow_visible && interface_prefs.compiler_tab_autoscroll)
 	{
-		path = gtk_tree_model_get_path(
+		GtkTreePath *path = gtk_tree_model_get_path(
 			gtk_tree_view_get_model(GTK_TREE_VIEW(msgwindow.tree_compiler)), &iter);
+
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(msgwindow.tree_compiler), path, NULL, TRUE, 0.5, 0.5);
 		gtk_tree_path_free(path);
 	}
@@ -320,25 +392,29 @@ void msgwin_show_hide(gboolean show)
 		GTK_CHECK_MENU_ITEM(ui_lookup_widget(main_widgets.window, "menu_show_messages_window1")),
 		show);
 	ignore_callback = FALSE;
-	ui_widget_show_hide(ui_lookup_widget(main_widgets.window, "scrolledwindow1"), show);
+	ui_widget_show_hide(main_widgets.message_window_notebook, show);
 	/* set the input focus back to the editor */
 	keybindings_send_command(GEANY_KEY_GROUP_FOCUS, GEANY_KEYS_FOCUS_EDITOR);
 }
 
 
 /**
- *  Adds a new message in the messages tab treeview in the messages window.
- *  If @a line and @a doc are set, clicking on this line jumps into the file which is specified
- *  by @a doc into the line specified with @a line.
+ * Adds a formatted message in the messages tab treeview in the messages window.
  *
- *  @param msg_color A color to be used for the text. It must be an element of #MsgColors.
- *  @param line The document's line where the message belongs to. Set to @c -1 to ignore.
- *  @param doc The document. Set to @c NULL to ignore.
- *  @param format @c printf()-style format string.
- *  @param ... Arguments for the @c format string.
+ * If @a line and @a doc are set, clicking on this line jumps into the file
+ * which is specified by @a doc into the line specified with @a line.
  *
- * @since 0.15
+ * @param msg_color A color to be used for the text. It must be an element of #MsgColors.
+ * @param line      The document's line where the message belongs to. Set to @c -1 to ignore.
+ * @param doc       @nullable The document. Set to @c NULL to ignore.
+ * @param format    @c printf()-style format string.
+ * @param ...       Arguments for the @c format string.
+ *
+ * @see msgwin_msg_add_string()
+ *
+ * @since 0.16
  **/
+GEANY_API_SYMBOL
 void msgwin_msg_add(gint msg_color, gint line, GeanyDocument *doc, const gchar *format, ...)
 {
 	gchar *string;
@@ -353,7 +429,22 @@ void msgwin_msg_add(gint msg_color, gint line, GeanyDocument *doc, const gchar *
 }
 
 
-/* adds string to the msg treeview */
+/**
+ * Adds a new message in the messages tab treeview in the messages window.
+ *
+ * If @a line and @a doc are set, clicking on this line jumps into the
+ * file which is specified by @a doc into the line specified with @a line.
+ *
+ * @param msg_color A color to be used for the text. It must be an element of #MsgColors.
+ * @param line      The document's line where the message belongs to. Set to @c -1 to ignore.
+ * @param doc       @nullable The document. Set to @c NULL to ignore.
+ * @param string    Message to be added.
+ *
+ * @see msgwin_msg_add()
+ *
+ * @since 1.34 (API 236)
+ **/
+GEANY_API_SYMBOL
 void msgwin_msg_add_string(gint msg_color, gint line, GeanyDocument *doc, const gchar *string)
 {
 	GtkTreeIter iter;
@@ -380,7 +471,9 @@ void msgwin_msg_add_string(gint msg_color, gint line, GeanyDocument *doc, const
 		utf8_msg = tmp;
 
 	gtk_list_store_append(msgwindow.store_msg, &iter);
-	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, doc, 2, color, 3, utf8_msg, -1);
+	gtk_list_store_set(msgwindow.store_msg, &iter,
+		MSG_COL_LINE, line, MSG_COL_DOC_ID, doc ? doc->id : 0, MSG_COL_COLOR,
+		color, MSG_COL_STRING, utf8_msg, -1);
 
 	g_free(tmp);
 	if (utf8_msg != tmp)
@@ -389,28 +482,26 @@ void msgwin_msg_add_string(gint msg_color, gint line, GeanyDocument *doc, const
 
 
 /**
- *  Logs a status message *without* setting the status bar.
- *  (Use ui_set_statusbar() to display text on the statusbar)
+ * Logs a new status message *without* setting the status bar.
  *
- *  @param format @c printf()-style format string.
- *  @param ... Arguments for the @c format string.
+ * Use @ref ui_set_statusbar() to display text on the statusbar.
+ *
+ * @param string Status message to be logged.
+ *
+ * @see msgwin_status_add()
+ *
+ * @since 1.34 (API 236)
  **/
-void msgwin_status_add(const gchar *format, ...)
+GEANY_API_SYMBOL
+void msgwin_status_add_string(const gchar *string)
 {
 	GtkTreeIter iter;
-	gchar *string;
 	gchar *statusmsg, *time_str;
-	va_list args;
-
-	va_start(args, format);
-	string = g_strdup_vprintf(format, args);
-	va_end(args);
 
 	/* add a timestamp to status messages */
 	time_str = utils_get_current_time_string();
 	statusmsg = g_strconcat(time_str, ": ", string, NULL);
 	g_free(time_str);
-	g_free(string);
 
 	/* add message to Status window */
 	gtk_list_store_append(msgwindow.store_status, &iter);
@@ -428,6 +519,32 @@ void msgwin_status_add(const gchar *format, ...)
 	}
 }
 
+/**
+ * Logs a formatted status message *without* setting the status bar.
+ *
+ * Use @ref ui_set_statusbar() to display text on the statusbar.
+ *
+ * @param format @c printf()-style format string.
+ * @param ...    Arguments for the @c format string.
+ *
+ * @see msgwin_status_add_string()
+ *
+ * @since 0.12
+ **/
+GEANY_API_SYMBOL
+void msgwin_status_add(const gchar *format, ...)
+{
+	gchar *string;
+	va_list args;
+
+	va_start(args, format);
+	string = g_strdup_vprintf(format, args);
+	va_end(args);
+
+	msgwin_status_add_string(string);
+	g_free(string);
+}
+
 
 static void
 on_message_treeview_clear_activate(GtkMenuItem *menuitem, gpointer user_data)
@@ -445,7 +562,7 @@ on_compiler_treeview_copy_activate(GtkMenuItem *menuitem, gpointer user_data)
 	GtkTreeSelection *selection;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	gint str_idx = 1;
+	gint str_idx = COMPILER_COL_STRING;
 
 	switch (GPOINTER_TO_INT(user_data))
 	{
@@ -460,7 +577,7 @@ on_compiler_treeview_copy_activate(GtkMenuItem *menuitem, gpointer user_data)
 
 		case MSG_MESSAGE:
 		tv = msgwindow.tree_msg;
-		str_idx = 3;
+		str_idx = MSG_COL_STRING;
 		break;
 	}
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv));
@@ -485,7 +602,7 @@ static void on_compiler_treeview_copy_all_activate(GtkMenuItem *menuitem, gpoint
 	GtkListStore *store = msgwindow.store_compiler;
 	GtkTreeIter iter;
 	GString *str = g_string_new("");
-	gint str_idx = 1;
+	gint str_idx = COMPILER_COL_STRING;
 	gboolean valid;
 
 	switch (GPOINTER_TO_INT(user_data))
@@ -501,7 +618,7 @@ static void on_compiler_treeview_copy_all_activate(GtkMenuItem *menuitem, gpoint
 
 		case MSG_MESSAGE:
 		store = msgwindow.store_msg;
-		str_idx = 3;
+		str_idx = MSG_COL_STRING;
 		break;
 	}
 
@@ -611,7 +728,7 @@ find_prev_build_dir(GtkTreePath *cur, GtkTreeModel *model, gchar **prefix)
 		if (gtk_tree_model_get_iter(model, &iter, cur))
 		{
 			gchar *string;
-			gtk_tree_model_get(model, &iter, 1, &string, -1);
+			gtk_tree_model_get(model, &iter, COMPILER_COL_STRING, &string, -1);
 			if (string != NULL && build_parse_make_dir(string, prefix))
 			{
 				g_free(string);
@@ -625,11 +742,16 @@ find_prev_build_dir(GtkTreePath *cur, GtkTreeModel *model, gchar **prefix)
 }
 
 
-static gboolean goto_compiler_file_line(const gchar *filename, gint line, gboolean focus_editor)
+static gboolean goto_compiler_file_line(const gchar *fname, gint line, gboolean focus_editor)
 {
-	if (!filename || line <= -1)
+	gboolean ret = FALSE;
+	gchar *filename;
+
+	if (!fname || line <= -1)
 		return FALSE;
 
+	filename = utils_get_locale_from_utf8(fname);
+
 	/* If the path doesn't exist, try the current document.
 	 * This happens when we receive build messages in the wrong order - after the
 	 * 'Leaving directory' messages */
@@ -649,8 +771,8 @@ static gboolean goto_compiler_file_line(const gchar *filename, gint line, gboole
 			if (g_file_test(name, G_FILE_TEST_EXISTS))
 			{
 				ui_set_statusbar(FALSE, _("Could not find file '%s' - trying the current document path."),
-					filename);
-				filename = name;
+					fname);
+				SETPTR(filename, name);
 			}
 			else
 				g_free(name);
@@ -669,8 +791,6 @@ static gboolean goto_compiler_file_line(const gchar *filename, gint line, gboole
 
 		if (doc != NULL)
 		{
-			gboolean ret;
-
 			if (! doc->changed && editor_prefs.use_indicators)	/* if modified, line may be wrong */
 				editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line - 1);
 
@@ -678,10 +798,13 @@ static gboolean goto_compiler_file_line(const gchar *filename, gint line, gboole
 			if (ret && focus_editor)
 				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 
-			return ret;
+			ret = TRUE;
 		}
 	}
-	return FALSE;
+
+	g_free(filename);
+
+	return ret;
 }
 
 
@@ -697,7 +820,7 @@ gboolean msgwin_goto_compiler_file_line(gboolean focus_editor)
 	if (gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
 		/* if the item is not coloured red, it's not an error line */
-		gtk_tree_model_get(model, &iter, 0, &color, -1);
+		gtk_tree_model_get(model, &iter, COMPILER_COL_COLOR, &color, -1);
 		if (color == NULL || ! gdk_color_equal(color, &color_error))
 		{
 			if (color != NULL)
@@ -706,7 +829,7 @@ gboolean msgwin_goto_compiler_file_line(gboolean focus_editor)
 		}
 		gdk_color_free(color);
 
-		gtk_tree_model_get(model, &iter, 1, &string, -1);
+		gtk_tree_model_get(model, &iter, COMPILER_COL_STRING, &string, -1);
 		if (string != NULL)
 		{
 			gint line;
@@ -748,7 +871,7 @@ static void make_absolute(gchar **filename, const gchar *dir)
 }
 
 
-/* try to parse the file and line number where the error occured described in line
+/* try to parse the file and line number where the error occurred described in line
  * and when something useful is found, it stores the line number in *line and the
  * relevant file with the error in *filename.
  * *line will be -1 if no error was found in string.
@@ -957,7 +1080,7 @@ static void parse_compiler_error_line(const gchar *string,
 				data.file_idx = 1;
 				break;
 			}
-			/* don't accidently find libtool versions x:y:x and think it is a file name */
+			/* don't accidentally find libtool versions x:y:x and think it is a file name */
 			if (strstr(string, "libtool --mode=link") == NULL)
 			{
 				data.pattern = ":";
@@ -974,7 +1097,7 @@ static void parse_compiler_error_line(const gchar *string,
 }
 
 
-/* try to parse the file and line number where the error occured described in string
+/* try to parse the file and line number where the error occurred described in string
  * and when something useful is found, it stores the line number in *line and the
  * relevant file with the error in *filename.
  * *line will be -1 if no error was found in string.
@@ -983,7 +1106,7 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
 		gchar **filename, gint *line)
 {
 	GeanyFiletype *ft;
-	gchar *trimmed_string;
+	gchar *trimmed_string, *utf8_dir;
 
 	*filename = NULL;
 	*line = -1;
@@ -992,8 +1115,10 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
 		return;
 
 	if (dir == NULL)
-		dir = build_info.dir;
-	g_return_if_fail(dir != NULL);
+		utf8_dir = utils_get_utf8_from_locale(build_info.dir);
+	else
+		utf8_dir = g_strdup(dir);
+	g_return_if_fail(utf8_dir != NULL);
 
 	trimmed_string = g_strdup(string);
 	g_strchug(trimmed_string); /* remove possible leading whitespace */
@@ -1006,8 +1131,9 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
 		/* fallback to default old-style parsing */
 		parse_compiler_error_line(trimmed_string, filename, line);
 	}
-	make_absolute(filename, dir);
+	make_absolute(filename, utf8_dir);
 	g_free(trimmed_string);
+	g_free(utf8_dir);
 }
 
 
@@ -1026,7 +1152,7 @@ static void msgwin_parse_generic_line(const gchar *string, gchar **filename, gin
 	/* extract the filename */
 	if (fields[0] != NULL)
 	{
-		*filename = g_strdup(fields[0]);
+		*filename = utils_get_locale_from_utf8(fields[0]);
 		if (msgwindow.messages_dir != NULL)
 			make_absolute(filename, msgwindow.messages_dir);
 
@@ -1067,17 +1193,28 @@ gboolean msgwin_goto_messages_file_line(gboolean focus_editor)
 	if (gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
 		gint line;
+		guint id;
 		gchar *string;
 		GeanyDocument *doc;
 		GeanyDocument *old_doc = document_get_current();
 
-		gtk_tree_model_get(model, &iter, 0, &line, 1, &doc, 3, &string, -1);
-		/* doc may have been closed, so check doc->index: */
-		if (line >= 0 && DOC_VALID(doc))
+		gtk_tree_model_get(model, &iter,
+			MSG_COL_LINE, &line, MSG_COL_DOC_ID, &id, MSG_COL_STRING, &string, -1);
+		if (line >= 0 && id > 0)
 		{
-			ret = navqueue_goto_line(old_doc, doc, line);
-			if (ret && focus_editor)
-				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
+			/* check doc is still open */
+			doc = document_find_by_id(id);
+			if (!doc)
+			{
+				ui_set_statusbar(FALSE, _("The document has been closed."));
+				utils_beep();
+			}
+			else
+			{
+				ret = navqueue_goto_line(old_doc, doc, line);
+				if (ret && focus_editor)
+					gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
+			}
 		}
 		else if (line < 0 && string != NULL)
 		{
@@ -1157,15 +1294,17 @@ static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *
 
 
 /**
- *  Switches to the given notebook tab of the messages window and shows the messages window
- *  if it was previously hidden and @a show is set to @c TRUE.
+ * Switches to the given notebook tab of the messages window.
  *
- *  @param tabnum An index of a tab in the messages window. Valid values are all elements of
- *                #MessageWindowTabNum.
- *  @param show Whether to show the messages window at all if it was hidden before.
+ * The messages window is shown if it was previously hidden and @a show is set to @c TRUE.
+ *
+ * @param tabnum An index of a tab in the messages window. Valid values are
+ *                all elements of #MessageWindowTabNum.
+ * @param show   Whether to show the messages window at all if it was hidden before.
  *
  * @since 0.15
  **/
+GEANY_API_SYMBOL
 void msgwin_switch_tab(gint tabnum, gboolean show)
 {
 	GtkWidget *widget = NULL;	/* widget to focus */
@@ -1193,13 +1332,14 @@ void msgwin_switch_tab(gint tabnum, gboolean show)
 
 
 /**
- *  Removes all messages from a tab specified by @a tabnum in the messages window.
+ * Removes all messages from a tab specified by @a tabnum in the messages window.
  *
- *  @param tabnum An index of a tab in the messages window which should be cleared.
+ * @param tabnum An index of a tab in the messages window which should be cleared.
  *                Valid values are @c MSG_STATUS, @c MSG_COMPILER and @c MSG_MESSAGE.
  *
  * @since 0.15
  **/
+GEANY_API_SYMBOL
 void msgwin_clear_tab(gint tabnum)
 {
 	GtkListStore *store = NULL;
